<?php
/**
 * EyemagineHubspot\Inc\Api\DataSync\EYEHUB_Abandoned
 *
 * @package  EyemagineHubspot
 */

namespace EyemagineHubspot\Inc\Api\DataSync;

use EyemagineHubspot\Inc\Api\EYEHUB_Data_Sync;
use EyemagineHubspot\Inc\Base\EYEHUB_Activate;

/**
 * EYEHUB_Abandoned Class
 */
class EYEHUB_Abandoned extends EYEHUB_Data_Sync {

	/**
	 * Get Data.
	 */
	public function eyehub_get_data() {
		global $wpdb;
		$stores     = array();
		$start      = 0;
		$start_date = isset( $_GET['start_date'] ) && absint( $_GET['start_date'] ) ? gmdate( 'Y-m-d H:m:i', absint( $_GET['start_date'] ) ) : null;
		$count      = empty( $_GET['count'] ) ? 50 : absint( $_GET['count'] );
		if ( ! $start_date ) {
			$start = empty( $_GET['start'] ) ? 0 : absint( $_GET['start'] );
		}

		try {

			$this->eyehub_authenticate();

			// If it has not already been done, update usermeta table.
			EYEHUB_Activate::eyehub_update_usermeta_table();

			// Prepare MySQL statements to collect data.
			$abandoned_cart_data = json_decode(
				wp_json_encode(
					$wpdb->get_results(
						$wpdb->prepare(
							"SELECT * FROM  {$wpdb->usermeta} WHERE umeta_id > %d AND meta_key = %s AND updated_at > %s ORDER BY umeta_id ASC LIMIT %d",
							$start,
							'_woocommerce_persistent_cart_' . get_current_blog_id(),
							$start_date,
							$count
						)
					)
				),
				true
			);

			// prepare data for response.
			foreach ( $abandoned_cart_data as $key => $cart_data ) {

				// unserialize and check if there are products in cart.
				$cart_items = unserialize( $cart_data['meta_value'] );
				if ( ! count( $cart_items['cart'] ) ) {
					unset( $abandoned_cart_data[ $key ] );
					continue;
				}
				if ( $start_date && ( strtotime( $cart_data['updated_at'] ) ) < $start_date ) {
					unset( $abandoned_cart_data[ $key ] );
					continue;
				}
				$cart = array();
				// prepare abandoned cart data.
				foreach ( $cart_items['cart'] as $item ) {
					$cart[] = $item;
				}

				$abandoned_cart_data[ $key ]['meta_value'] = $cart;

				// get products data.
				if ( ! empty( $cart ) ) {
					$items = array();
					foreach ( $cart as $value ) {
						if ( isset( $value['product_id'] ) && wc_get_product( $value['product_id'] ) ) {
							$product      = wc_get_product( $value['product_id'] );
							$product_data = $product->get_data();
							if ( ! empty( $product_data['image_id'] ) ) {
								$product_data['image'] = wp_get_attachment_image_src( $product_data['image_id'] )[0];
							}
							if ( ! empty( $product_data['category_ids'] ) ) {
								foreach ( $product_data['category_ids'] as $cat ) {
									$category                     = get_term_by( 'id', $cat, 'product_cat' );
									$product_data['categories'][] = $category;
								}
							}
							if ( ! empty( $product_data['upsell_ids'] ) ) {
								$upsell_product_data = array();
								foreach ( $product_data['upsell_ids'] as $upsell_id ) {
									$upsell_product = wc_get_product( $upsell_id );
									if ( $upsell_product ) {
										$upsell_product_data[] = $upsell_product->get_data();
									}
								}
								$product_data['up_sell_products'] = $upsell_product_data;
							}
							if ( ! empty( $product_data['cross_sell_ids'] ) ) {
								$cross_sell_product_data = array();
								foreach ( $product_data['cross_sell_ids'] as $cross_sell_id ) {
									$cross_sell_product = wc_get_product( $cross_sell_id );
									if ( $cross_sell_product ) {
										$cross_sell_product_data[] = $cross_sell_product->get_data();
									}
								}
								$product_data['cross_sell_products'] = $cross_sell_product_data;
							}

							$product_data['upsell_ids']     = ( ! empty( $product_data['upsell_ids'] ) ) ? implode( ',', $product_data['upsell_ids'] ) : null;
							$product_data['cross_sell_ids'] = ( ! empty( $product_data['cross_sell_ids'] ) ) ? implode( ',', $product_data['cross_sell_ids'] ) : null;
							$product_data['category_ids']   = ( ! empty( $product_data['category_ids'] ) ) ? implode( ',', $product_data['category_ids'] ) : null;
							$product_data['quantity']       = ( ! empty( $value['quantity'] ) ) ? $value['quantity'] : null;
							$product_data['product']        = $product_data;
						}
						if ( !empty($product_data) ) {
							$items[] = $product_data;
						}
					}
					$abandoned_cart_data[ $key ]['items'] = $items;
				}

				// get user meta data.
				$user_meta_data = get_user_meta( $abandoned_cart_data[ $key ]['user_id'] );
				$user_full_data = get_userdata( $abandoned_cart_data[ $key ]['user_id'] );
				$user_data      = array();

				// gather user data in 1 array.
				foreach ( $user_meta_data as $meta_key => $meta_val ) {
					if ( isset( $meta_val[0] ) ) {
						$user_data[ $meta_key ] = $meta_val[0];
					} else {
						$user_data[ $meta_key ] = $meta_val;
					}
				}

				// data mapping.
				$mappeduser_data                   = array();
				$mappeduser_data['entity_id']      = isset( $user_data['ID'] ) ? $user_data['ID'] : $user_full_data->ID;
				$mappeduser_data['email']          = isset( $user_data['user_email'] ) ? $user_data['user_email'] : $user_full_data->user_email;
				$mappeduser_data['store_id']       = get_current_blog_id();
				$mappeduser_data['created_at']     = isset( $user_data['user_registered'] ) ? $user_data['user_registered'] : $user_full_data->user_registered;
				$mappeduser_data['updated_at']     = isset( $user_data['last_update'] ) ? gmdate( 'Y-m-d H:i:s', $user_data['last_update'] ) : $mappeduser_data['created_at'];
				$mappeduser_data['is_active']      = isset( $user_data['user_status'] ) ? $user_data['user_status'] : $user_full_data->user_status;
				$mappeduser_data['firstname']      = isset( $user_data['first_name'] ) ? $user_data['first_name'] : '';
				$mappeduser_data['lastname']       = isset( $user_data['last_name'] ) ? $user_data['last_name'] : '';
				$mappeduser_data['customer_group'] = implode( ',', $user_full_data->roles );
				$mappeduser_data['nickname']       = isset( $user_data['nickname'] ) ? $user_data['nickname'] : $user_full_data->nickname;
				$mappeduser_data['display_name']   = isset( $user_data['display_name'] ) ? $user_data['nickname'] : $user_full_data->display_name;

				// billing address mapping.
				if ( ! empty( $user_data['billing_first_name'] ) && ! empty( $user_data['billing_address_1'] ) ) {
					$mapped_user_data['billing'] = array(
						'first_name' => isset( $user_data['billing_first_name'] ) ? $user_data['billing_first_name'] : '',
						'last_name'  => isset( $user_data['billing_last_name'] ) ? $user_data['billing_last_name'] : '',
						'company'    => isset( $user_data['billing_company'] ) ? $user_data['billing_company'] : '',
						'address_1'  => isset( $user_data['billing_address_1'] ) ? $user_data['billing_address_1'] : '',
						'address_2'  => isset( $user_data['billing_address_2'] ) ? $user_data['billing_address_2'] : '',
						'city'       => isset( $user_data['billing_city'] ) ? $user_data['billing_city'] : '',
						'postcode'   => isset( $user_data['billing_postcode'] ) ? $user_data['billing_postcode'] : '',
						'state'      => isset( $user_data['billing_state'] ) ? $user_data['billing_state'] : '',
						'country'    => isset( $user_data['billing_country'] ) ? $user_data['billing_country'] : '',
						'email'      => isset( $user_data['billing_email'] ) ? $user_data['billing_email'] : '',
						'phone'      => isset( $user_data['billing_phone'] ) ? $user_data['billing_phone'] : '',
					);
				}
				// shipping address mapping.
				if ( ! empty( $user_data['shipping_first_name'] ) && ! empty( $user_data['shipping_address_1'] ) ) {
					$mapped_user_data['shipping'] = array(
						'first_name' => isset( $user_data['shipping_first_name'] ) ? $user_data['shipping_first_name'] : '',
						'last_name'  => isset( $user_data['shipping_last_name'] ) ? $user_data['shipping_last_name'] : '',
						'company'    => isset( $user_data['shipping_company'] ) ? $user_data['shipping_company'] : '',
						'address_1'  => isset( $user_data['shipping_address_1'] ) ? $user_data['shipping_address_1'] : '',
						'address_2'  => isset( $user_data['shipping_address_2'] ) ? $user_data['shipping_address_2'] : '',
						'city'       => isset( $user_data['shipping_city'] ) ? $user_data['shipping_city'] : '',
						'postcode'   => isset( $user_data['shipping_postcode'] ) ? $user_data['shipping_postcode'] : '',
						'state'      => isset( $user_data['shipping_state'] ) ? $user_data['shipping_state'] : '',
						'country'    => isset( $user_data['shipping_country'] ) ? $user_data['shipping_country'] : '',
					);
				}

				$abandoned_cart_data[ $key ]['user_data'] = $mappeduser_data;
			}
		} catch ( Exception $e ) {
			$this->eyehub_output_error(
				self::EYEHUB_ERROR_CODE_UNKNOWN_EXCEPTION,
				'Unknown exception on request',
				$e
			);

			return;
		}

		$this->eyehub_output_json(
			array(
				'abandoned' => $abandoned_cart_data,
				'stores'    => $stores,
				'blogId'    => get_current_blog_id(),
				'start'     => $start,
			)
		);

	}
}
