<?php
/**
 * EyemagineHubspot\Inc\Api\DataSync\EYEHUB_Customers
 *
 * @package  EyemagineHubspot
 */

namespace EyemagineHubspot\Inc\Api\DataSync;

use EyemagineHubspot\Inc\Api\EYEHUB_Data_Sync;

/**
 * EYEHUB_Customers Class
 */
class EYEHUB_Customers extends EYEHUB_Data_Sync {

	/**
	 * Get Data.
	 */
	public function eyehub_get_data() {
		global $wpdb;
		$users_data = array();

		$stores     = array();
		$start      = isset( $_GET['start'] ) && absint( $_GET['start'] ) ? absint( $_GET['start'] ) : 0;
		$count      = isset( $_GET['count'] ) && absint( $_GET['count'] ) ? absint( $_GET['count'] ) : 50;
		$start_date = isset( $_GET['start_date'] ) && absint( $_GET['start_date'] ) ? absint( $_GET['start_date'] ) : null;

		try {
			$this->eyehub_authenticate();

			if ( $start_date ) {
				$users = get_users(
					array(
						'blog_id'      => get_current_blog_id(),
						'meta_key'     => 'last_update',
						'meta_value'   => $start_date,
						'meta_compare' => '>',
						'role'         => 'customer',
						'number'       => $count,
						'count_total'  => false,
						'orderby'      => 'last_update',
						'order'        => 'ASC',
					)
				);
			} else {
				$users = get_users(
					array(
						'blog_id'     => get_current_blog_id(),
						'role'        => 'customer',
						'exclude'     => $start ? range( 0, $start ) : array(),
						'orderby'     => 'id',
						'order'       => 'ASC',
						'number'      => $count,
						'paged'       => 1,
						'count_total' => false,
						'fields'      => 'all',
					)
				);
			}

			foreach ( $users as $user ) {
				$user_data = array();
				// fetch user's meta data.
				$user_meta_data = get_user_meta( $user->ID );

				// gather user data in 1 array.
				foreach ( $user_meta_data as $meta_key => $meta_val ) {
					if ( isset( $meta_val[0] ) ) {
						$user_data[ $meta_key ] = $meta_val[0];
					} else {
						$user_data[ $meta_key ] = $meta_val;
					}
				}

				// data mapping.
				$mapped_user_data                   = array();
				$mapped_user_data['entity_id']      = $user->ID;
				$mapped_user_data['email']          = $user->user_email;
				$mapped_user_data['created_at']     = isset( $user->user_registered ) ? $user->user_registered : gmdate( 'Y-m-d H:i:s' );
				$mapped_user_data['updated_at']     = isset( $user->last_update ) ? gmdate( 'Y-m-d H:i:s', $user->last_update ) : $mapped_user_data['created_at'];
				$mapped_user_data['is_active']      = isset( $user->user_status ) ? $user->user_status : 0;
				$mapped_user_data['firstname']      = isset( $user_data['first_name'] ) ? $user_data['first_name'] : '';
				$mapped_user_data['lastname']       = isset( $user_data['last_name'] ) ? $user_data['last_name'] : '';
				$mapped_user_data['customer_group'] = implode( ',', $user->roles );
				$mapped_user_data['nickname']       = isset( $user->nickname ) ? $user->nickname : '';
				$mapped_user_data['display_name']   = isset( $user->display_name ) ? $user->display_name : '';

				// billing address mapping.
				if ( ! empty( $user_data['billing_first_name'] ) && ! empty( $user_data['billing_address_1'] ) ) {
					$mapped_user_data['billing'] = array(
						'first_name' => isset( $user_data['billing_first_name'] ) ? $user_data['billing_first_name'] : '',
						'last_name'  => isset( $user_data['billing_last_name'] ) ? $user_data['billing_last_name'] : '',
						'company'    => isset( $user_data['billing_company'] ) ? $user_data['billing_company'] : '',
						'address_1'  => isset( $user_data['billing_address_1'] ) ? $user_data['billing_address_1'] : '',
						'address_2'  => isset( $user_data['billing_address_2'] ) ? $user_data['billing_address_2'] : '',
						'city'       => isset( $user_data['billing_city'] ) ? $user_data['billing_city'] : '',
						'postcode'   => isset( $user_data['billing_postcode'] ) ? $user_data['billing_postcode'] : '',
						'state'      => isset( $user_data['billing_state'] ) ? $user_data['billing_state'] : '',
						'country'    => isset( $user_data['billing_country'] ) ? $user_data['billing_country'] : '',
						'email'      => isset( $user_data['billing_email'] ) ? $user_data['billing_email'] : '',
						'phone'      => isset( $user_data['billing_phone'] ) ? $user_data['billing_phone'] : '',
					);
				}
				// shipping address mapping.
				if ( ! empty( $user_data['shipping_first_name'] ) && ! empty( $user_data['shipping_address_1'] ) ) {
					$mapped_user_data['shipping'] = array(
						'first_name' => isset( $user_data['shipping_first_name'] ) ? $user_data['shipping_first_name'] : '',
						'last_name'  => isset( $user_data['shipping_last_name'] ) ? $user_data['shipping_last_name'] : '',
						'company'    => isset( $user_data['shipping_company'] ) ? $user_data['shipping_company'] : '',
						'address_1'  => isset( $user_data['shipping_address_1'] ) ? $user_data['shipping_address_1'] : '',
						'address_2'  => isset( $user_data['shipping_address_2'] ) ? $user_data['shipping_address_2'] : '',
						'city'       => isset( $user_data['shipping_city'] ) ? $user_data['shipping_city'] : '',
						'postcode'   => isset( $user_data['shipping_postcode'] ) ? $user_data['shipping_postcode'] : '',
						'state'      => isset( $user_data['shipping_state'] ) ? $user_data['shipping_state'] : '',
						'country'    => isset( $user_data['shipping_country'] ) ? $user_data['shipping_country'] : '',
					);
				}
				$users_data[ $user->ID ] = $mapped_user_data;
			}
		} catch ( \Exception $e ) {
			$this->eyehub_output_error(
				self::EYEHUB_ERROR_CODE_UNKNOWN_EXCEPTION,
				'Unknown exception on request',
				$e
			);

			return;
		}

		$this->eyehub_output_json(
			array(
				'customers' => $users_data,
				'blogId'    => get_current_blog_id(),
				'store'     => $stores,
				'start'     => $start,
			)
		);
	}
}
