<?php
/**
 * EyemagineHubspot\Inc\Api\DataSync\EYEHUB_Orders
 *
 * @package  EyemagineHubspot
 */

namespace EyemagineHubspot\Inc\Api\DataSync;

use EyemagineHubspot\Inc\Api\EYEHUB_Data_Sync;

/**
 * EYEHUB_Orders Class
 */
class EYEHUB_Orders extends EYEHUB_Data_Sync {

	/**
	 * Get Data.
	 */
	public function eyehub_get_data() {
		global $wpdb;
		$orders_data = array();
		$orders      = null;
		$stores      = array();
		$start       = isset( $_GET['start'] ) && absint( $_GET['start'] ) ? absint( $_GET['start'] ) : 0;
		$count       = isset( $_GET['count'] ) && absint( $_GET['count'] ) ? absint( $_GET['count'] ) : 50;
		$start_date  = isset( $_GET['start_date'] ) && absint( $_GET['start_date'] ) ? absint( $_GET['start_date'] ) : null;

		try {

			$this->eyehub_authenticate();

			if ( $start_date ) {
				$orders = get_posts(
					array(
						'post_type'   => wc_get_order_types(),
						'post_status' => array_keys( wc_get_order_statuses() ),
						'numberposts' => $count,
						'date_query'  => array(
							array(
								'column'    => 'post_modified',
								'after'     => gmdate( 'Y-m-d', $start_date ),
								'inclusive' => true,
							),
							array(
								'column'  => 'post_modified',
								'hour'    => (int) gmdate( 'H', $start_date ),
								'compare' => '>=',
							),
							array(
								'column'  => 'post_modified',
								'minute'  => (int) gmdate( 'i', $start_date ),
								'compare' => '>=',
							),
							array(
								'column'  => 'post_modified',
								'second'  => (int) gmdate( 's', $start_date ),
								'compare' => '>=',
							),
						),
						'orderby'     => 'post_modified',
						'order'       => 'ASC',
					)
				);
			} else {
				$orders = get_posts(
					array(
						'post_type'   => wc_get_order_types(),
						'post_status' => array_keys( wc_get_order_statuses() ),
						'exclude'     => $start ? range( 0, $start ) : array(),
						'numberposts' => $count,
						'orderby'     => 'ID',
						'order'       => 'ASC',
					)
				);
			}

			// prepare data for response.
			foreach ( $orders as $order ) {

				// Get an instance of the WC_Order object (same as before).
				$order = wc_get_order( $order->ID );

				// sanity check.
				if ( ! is_object( $order ) ) {
					continue;
				}

				// prepare order data.
				$order_data = $order->get_data();
				foreach ( $order->get_items() as $item ) {
					$order_item = $item->get_data();
					if ( $order_item['product_id'] && wc_get_product( $order_item['product_id'] ) ) {
						$product      = wc_get_product( $order_item['product_id'] );
						$product_data = $product->get_data();
						if ( ! empty( $product_data['image_id'] ) ) {
							$product_data['image'] = wp_get_attachment_image_src( $product_data['image_id'] )[0];
						}
						if ( ! empty( $product_data['category_ids'] ) ) {
							foreach ( $product_data['category_ids'] as $cat ) {
								$category                   = get_term_by( 'id', $cat, 'product_cat' );
								$order_item['categories'][] = $category;
							}
						}
						if ( ! empty( $product_data['upsell_ids'] ) ) {
							$upsell_product_data = array();
							foreach ( $product_data['upsell_ids'] as $upsell_id ) {
								$upsell_product = wc_get_product( $upsell_id );
								if ( $upsell_product ) {
									$upsell_product_data[] = $upsell_product->get_data();
								}
							}
							$product_data['up_sell_products'] = $upsell_product_data;
						}
						if ( ! empty( $product_data['cross_sell_ids'] ) ) {
							$cross_sell_product_data = array();
							foreach ( $product_data['cross_sell_ids'] as $cross_sell_id ) {
								$cross_sell_product = wc_get_product( $cross_sell_id );
								if ( $cross_sell_product ) {
									$cross_sell_product_data[] = $cross_sell_product->get_data();
								}
							}
							$product_data['cross_sell_products'] = $cross_sell_product_data;
						}

						$order_item['product']        = $product_data;
						$order_item['upsell_ids']     = ( ! empty( $product_data['upsell_ids'] ) ) ? implode( ',', $product_data['upsell_ids'] ) : null;
						$order_item['cross_sell_ids'] = ( ! empty( $product_data['cross_sell_ids'] ) ) ? implode( ',', $product_data['cross_sell_ids'] ) : null;
						$order_item['children_ids']   = ( ! empty( $product_data['children'] ) ) ? implode( ',', $product_data['children'] ) : null;
						$order_item['category_ids']   = ( ! empty( $product_data['category_ids'] ) ) ? implode( ',', $product_data['category_ids'] ) : null;
					}
					if ( $order_item ) {
						$order_data['order_items'][] = $order_item;
					}
				}
				$orders_data[] = $order_data;
			}
		} catch ( \Exception $e ) {
			$this->eyehub_output_error(
				self::EYEHUB_ERROR_CODE_UNKNOWN_EXCEPTION,
				'Unknown exception on request',
				$e
			);

			return;
		}

		$this->eyehub_output_json(
			array(
				'orders' => $orders_data,
				'stores' => $stores,
				'blogId' => get_current_blog_id(),
				'start'  => $start,
			)
		);

	}
}
