<?php
/**
 * EyemagineHubspot\Inc\Api\EYEHUB_Data_Sync
 *
 * @package  EyemagineHubspot
 */

namespace EyemagineHubspot\Inc\Api;

/**
 * EYEHUB_Data_Sync Abstract Class
 */
abstract class EYEHUB_Data_Sync {
	/**
	 * API error codes
	 */
	const EYEHUB_ERROR_CODE_SECURE = 9001;

	const EYEHUB_ERROR_CODE_INVALID_STORECODE = 9003;

	const EYEHUB_ERROR_CODE_INVALID_REQUEST = 9004;

	const EYEHUB_ERROR_CODE_INVALID_CREDENTIALS = 9005;

	const EYEHUB_ERROR_CODE_UNKNOWN_EXCEPTION = 9006;

	/**
	 * Function responsible for collecting data
	 *
	 * @return mixed
	 */
	abstract public function eyehub_get_data();

	/**
	 * Verify that the requestor has valid access
	 *
	 * @return boolean
	 */
	protected function eyehub_authenticate() {
		$key1 = get_option( 'eyehub_user_key' ); // user key.
		$key2 = get_option( 'eyehub_access_code' ); // pass code.

		// check if credentials are valid.
		if ( empty( $key1 ) || empty( $key2 ) || empty( $_GET['ukey'] ) || empty( $_GET['code'] ) ||
			sanitize_text_field( $_GET['ukey'] ) !== $key1 || sanitize_text_field( $_GET['code'] ) !== $key2 ) {
			$this->eyehub_output_error( self::EYEHUB_ERROR_CODE_INVALID_CREDENTIALS, 'Invalid user key and/or access code' );
		}

		return true;
	}


	/**
	 * Function used to output an error and quit.
	 *
	 * @param string $code code.
	 * @param string $error error.
	 * @param Exception|null $e exception.
	 */
	public function eyehub_output_error(
		$code,
		$error,
		Exception $e = null
	) {
		wp_send_json(
			array(
				'error' => $error,
				'code'  => $code,
				'extra' => ( $e ) ? $e->getMessage() : '',
			)
		);
		die;
	}


	/**
	 * Flushes the data as JSON response
	 *
	 * @param mixed $data data.
	 */
	protected function eyehub_output_json( $data ) {
		wp_send_json( $data );
		die;
	}

}
