<?php
/**
 * EyemagineHubspot\Inc\Pages\EYEHUB_Admin
 *
 * @package  EyemagineHubspot
 */

namespace EyemagineHubspot\Inc\Pages;

use EyemagineHubspot\Inc\Api\EYEHUB_Settings_Api;
use EyemagineHubspot\Inc\Base\EYEHUB_Base_Controller;
use EyemagineHubspot\Inc\Api\Callbacks\EYEHUB_Admin_Callbacks;

/**
 * Class for handling admin related functions
 */
class EYEHUB_Admin extends EYEHUB_Base_Controller {
	/**
	 * Settings.
	 *
	 * @var object
	 */
	public $settings;

	/**
	 * Callbacks.
	 *
	 * @var object
	 */
	public $callbacks;

	/**
	 * Register.
	 */
	public function eyehub_register() {
		$this->settings  = new EYEHUB_Settings_Api();
		$this->callbacks = new EYEHUB_Admin_Callbacks();

		$this->eyehub_set_settings();
		$this->eyehub_set_sections();
		$this->eyehub_set_fields();

		add_action( 'admin_menu', array( $this, 'eyehub_add_admin_page' ) );
		add_action( 'admin_init', array( $this->settings, 'eyehub_register_custom_fields' ) );
		add_filter( 'plugin_action_links_' . EYEHUB_PLUGIN, array( $this, 'eyehub_admin_settings_link' ) );

	}

	/**
	 * Add admin page.
	 */
	public function eyehub_add_admin_page() {
		add_menu_page(
			'WooCommerce - HubSpot integrator',
			'HubSpot Integration',
			'manage_options',
			'eyemagine_hubspot_connector',
			array( $this, 'adminIndex' ),
			'dashicons-cart',
			57
		);
	}

	/**
	 * AdminIndex.
	 */
	public function adminIndex() {
		require_once EYEHUB_PLUGIN_PATH . 'templates/admin.php';
	}

	/**
	 * Shows settings link in admin panel
	 *
	 * @param array $links links.
	 *
	 * @return mixed
	 */
	public static function eyehub_admin_settings_link( $links ) {
		$settings_link = '<a href="' . admin_url( 'admin.php' ) . '?page=eyemagine_hubspot_connector">' . __( 'Settings', 'eyehub_settings' ) . '</a>';
		array_push( $links, $settings_link );

		return $links;
	}

	/**
	 * Set Settings.
	 */
	public function eyehub_set_settings() {
		$args = array(
			array(
				'option_group' => 'eyehub_options_group',
				'option_name'  => 'eyehub_active_status',
				'callback'     => array( $this->callbacks, 'eyehub_options_group' ),
			),
			array(
				'option_group' => 'eyehub_options_group',
				'option_name'  => 'eyehub_user_key_regen',
			),
			array(
				'option_group' => 'eyehub_options_group',
				'option_name'  => 'eyehub_access_code_regen',
			),
		);

		$this->settings->eyehub_set_settings( $args );
	}

	/**
	 * Set Sections.
	 */
	public function eyehub_set_sections() {
		$args = array(
			array(
				'id'       => 'eyehub_admin_index',
				'title'    => 'WooCommerce + HubSpot integration',
				'callback' => array( $this->callbacks, 'eyehub_admin_section' ),
				'page'     => 'eyehub_plugin',
			),
		);

		$this->settings->eyehub_set_sections( $args );
	}

	/**
	 * Set Fields.
	 */
	public function eyehub_set_fields() {

		require_once ABSPATH . 'wp-includes/pluggable.php';

		$args = array(
			array(
				'id'       => 'eyehub_active_status',
				'title'    => 'Activate/Deactivate',
				'callback' => array( $this->callbacks, 'eyehub_active_status_field' ),
				'page'     => 'eyehub_plugin',
				'section'  => 'eyehub_admin_index',
				'args'     => array(
					'label_for' => 'eyehub_active_status',
				),
			),
			array(
				'id'       => 'eyehub_user_key_regen',
				'title'    => 'User Key',
				'callback' => array( $this->callbacks, 'eyehub_user_key' ),
				'page'     => 'eyehub_plugin',
				'section'  => 'eyehub_admin_index',
			),
			array(
				'id'       => 'eyehub_access_code_regen',
				'title'    => 'Access Code',
				'callback' => array( $this->callbacks, 'eyehub_access_code' ),
				'page'     => 'eyehub_plugin',
				'section'  => 'eyehub_admin_index',
			),
		);

		$this->settings->eyehub_set_fields( $args );

		if ( isset( $_POST['eyehub_form'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['eyehub_form'] ) ), 'eyehub_options_form' ) ) {
			if ( isset( $_POST['eyehub_regen_user_key'] ) ) {
				update_option( 'eyehub_user_key', md5( gmdate( 'Y-m-d H:i:s', time() ) . wp_rand( 0, 32767 ) . 'eyemaginehub' ), true );
			}

			if ( isset( $_POST['eyehub_regen_access_code'] ) ) {
				update_option( 'eyehub_access_code', md5( ( wp_rand( 0, 32767 ) * ( 17 + wp_rand( 0, 100 ) ) ) . gmdate( 'Y-m-d H:i:s', time() ) . 'eyehubspot' ) );
			}
		}
	}

}
