<?php
/**
 * EyemagineHubspot\Inc\Pages\EYEHUB_Data_Sync
 *
 * @package  EyemagineHubspot
 */

namespace EyemagineHubspot\Inc\Pages;

use EyemagineHubspot\Inc\Base\EYEHUB_Base_Controller;
use EyemagineHubspot\Inc\Api\Callbacks\EYEHUB_Admin_Callbacks;

/**
 * Class for handling admin related functions
 */
class EYEHUB_Data_Sync extends EYEHUB_Base_Controller {
	/**
	 * Preset values definitions
	 */
	const EYEHUB_MAX_ORDER_PERPAGE = 20;

	const EYEHUB_MAX_CUSTOMER_PERPAGE = 100;

	const EYEHUB_IS_ABANDONED_IN_SECS = 3600;

	const EYEHUB_MAX_SUBSCRIBER_PERPAGE = 100;

	const EYEHUB_MAX_ASSOC_PRODUCT_LIMIT = 10;

	const EYEHUB_LONG_DATE_FORMAT = 'l, F j, Y \a\t h:i A \U\T\C';

	const EYEHUB_IS_MULTISTORE = false;

	const EYEHUB_VALID_API_ACTIONS = array(
		'getabandoned'   => \EyemagineHubspot\Inc\Api\DataSync\EYEHUB_Abandoned::class,
		'getcustomers'   => \EyemagineHubspot\Inc\Api\DataSync\EYEHUB_Customers::class,
		'getorders'      => \EyemagineHubspot\Inc\Api\DataSync\EYEHUB_Orders::class,
		'getsubscribers' => \EyemagineHubspot\Inc\Api\DataSync\EYEHUB_Subscribers::class,
		'getinfo'        => \EyemagineHubspot\Inc\Api\DataSync\EYEHUB_Info::class,
	);

	/**
	 * Register service for data synchronisation
	 */
	public function eyehub_register() {
		add_action( 'init', array( $this, 'eyehub_data_sync' ) );
	}

	/**
	 * Find which action needed to be performed and if it is among VALID_ACTIONS
	 * instantiate EYEHUB_Data_Sync class for that action
	 * call eyehub_get_data function for instantiated class
	 * example request URL: /eyehubspot?get=orders&key=1dfdd2456&code=efdf456wfwf
	 *
	 * @return mixed
	 */
	public function eyehub_data_sync() {

		$action  = isset( $_GET['get'] ) ? sanitize_text_field( $_GET['get'] ) : null;
		$request = ! empty( $_SERVER['REQUEST_URI'] ) ? sanitize_text_field( $_SERVER['REQUEST_URI'] ) : null;

		if ( stristr( $request, 'eyehubspot' ) ) {

			// check if action is valid.
			if ( ! array_key_exists( $action, self::EYEHUB_VALID_API_ACTIONS ) ) {
				wp_send_json( array( 'error' => 'Invalid action requested' ) );
				die;
			}

			// instantiate action class and get data.
			$action_class     = self::EYEHUB_VALID_API_ACTIONS[ $action ];
			$action_class_obj = new $action_class();
			$action_class_obj->eyehub_get_data();

		}

		return true;
	}
}
